"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _has2 = _interopRequireDefault(require("lodash/has"));

var _get2 = _interopRequireDefault(require("lodash/get"));

var _ui = require("@sanity/ui");

var PathUtils = _interopRequireWildcard(require("@sanity/util/paths"));

var _classnames = _interopRequireDefault(require("classnames"));

var _react = _interopRequireWildcard(require("react"));

var _reactAce = _interopRequireDefault(require("react-ace"));

var _changeIndicators = require("@sanity/base/lib/change-indicators");

var _patchEvent = require("part:@sanity/form-builder/patch-event");

var _default = _interopRequireDefault(require("part:@sanity/components/formfields/default"));

var _default2 = _interopRequireDefault(require("part:@sanity/components/fieldsets/default"));

var _createHighlightMarkers = _interopRequireDefault(require("./createHighlightMarkers"));

var _CodeInput = _interopRequireDefault(require("./CodeInput.css"));

var _config = require("./config");

require("brace/mode/batchfile");

require("brace/mode/css");

require("brace/mode/html");

require("brace/mode/typescript");

require("brace/mode/javascript");

require("brace/mode/json");

require("brace/mode/jsx");

require("brace/mode/markdown");

require("brace/mode/php");

require("brace/mode/sass");

require("brace/mode/scss");

require("brace/mode/python");

require("brace/mode/sh");

require("brace/mode/text");

require("./groq");

require("brace/theme/github");

require("brace/theme/monokai");

require("brace/theme/terminal");

require("brace/theme/tomorrow");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function compareNumbers(numA, numB) {
  return numA - numB;
} // Returns a string with the mode name if supported (because aliases), otherwise false


function isSupportedLanguage(mode) {
  var alias = _config.LANGUAGE_ALIASES[mode];

  if (alias) {
    return alias;
  }

  var isSupported = _config.SUPPORTED_LANGUAGES.find(lang => lang.value === mode);

  if (isSupported) {
    return mode;
  }

  return false;
}

class CodeInput extends _react.PureComponent {
  constructor() {
    super(...arguments);

    _defineProperty(this, "editor", null);

    _defineProperty(this, "_inputId", void 0);

    _defineProperty(this, "handleCodeChange", code => {
      var _this$props = this.props,
          type = _this$props.type,
          onChange = _this$props.onChange;
      var path = _config.PATH_CODE;
      var fixedLanguage = (0, _get2.default)(type, 'options.language');
      onChange(_patchEvent.PatchEvent.from([(0, _patchEvent.setIfMissing)({
        _type: type.name,
        language: fixedLanguage
      }), code ? (0, _patchEvent.set)(code, path) : (0, _patchEvent.unset)(path)]));
    });

    _defineProperty(this, "handleToggleSelectLine", lineNumber => {
      var _this$props2 = this.props,
          type = _this$props2.type,
          onChange = _this$props2.onChange,
          value = _this$props2.value;
      var path = ['highlightedLines'];
      var highlightedLines = value && value.highlightedLines || [];
      var position = highlightedLines.indexOf(lineNumber);
      var patches = [(0, _patchEvent.setIfMissing)({
        _type: type.name
      }), (0, _patchEvent.setIfMissing)([], ['highlightedLines'])];
      var addLine = position === -1;

      if (addLine) {
        // New element, figure out where to add it so it sorts correctly
        var sorted = highlightedLines.concat(lineNumber).sort(compareNumbers);
        position = sorted.indexOf(lineNumber);
        patches.push((0, _patchEvent.insert)([lineNumber], 'before', path.concat(position === sorted.length - 1 ? -1 : position)));
      } else if (highlightedLines.length === 1) {
        // Last element removed, unset whole path
        patches.push((0, _patchEvent.unset)(path)); // Temporary workaround for bug in react-ace
        // (https://github.com/securingsincity/react-ace/issues/229)

        var editor = this.editor // Remove all markers from editor
        ;
        [true, false].forEach(inFront => {
          var editorSession = editor === null || editor === void 0 ? void 0 : editor.getSession();
          var currentMarkers = editorSession.getMarkers(inFront);
          Object.keys(currentMarkers).forEach(marker => {
            editorSession.removeMarker(currentMarkers[marker].id);
          });
        });
      } else {
        // Removed, but not the last element, remove single item
        patches.push((0, _patchEvent.unset)(path.concat(position)));
      }

      onChange(_patchEvent.PatchEvent.from(patches));
    });

    _defineProperty(this, "handleGutterMouseDown", event => {
      var target = event.domEvent.target;

      if (target.classList.contains('ace_gutter-cell')) {
        var row = event.getDocumentPosition().row;
        this.handleToggleSelectLine(row + 1); // Ace starts at row 0
      }
    });

    _defineProperty(this, "handleEditorLoad", editor => {
      this.editor = editor;
      editor.focus();
      editor.on('guttermousedown', this.handleGutterMouseDown);
    });

    _defineProperty(this, "handleLanguageChange", event => {
      var _this$props3 = this.props,
          type = _this$props3.type,
          onChange = _this$props3.onChange;
      var value = event.currentTarget.value;
      var path = _config.PATH_LANGUAGE;
      onChange(_patchEvent.PatchEvent.from([(0, _patchEvent.setIfMissing)({
        _type: type.name
      }), value ? (0, _patchEvent.set)(value, path) : (0, _patchEvent.unset)(path)]));
    });

    _defineProperty(this, "handleFilenameChange", event => {
      var _this$props4 = this.props,
          type = _this$props4.type,
          onChange = _this$props4.onChange;
      var value = event.target.value;
      var path = _config.PATH_FILENAME;
      onChange(_patchEvent.PatchEvent.from([(0, _patchEvent.setIfMissing)({
        _type: type.name
      }), value ? (0, _patchEvent.set)(value, path) : (0, _patchEvent.unset)(path)]));
    });

    _defineProperty(this, "renderEditor", () => {
      var _this$props5 = this.props,
          readOnly = _this$props5.readOnly,
          value = _this$props5.value,
          type = _this$props5.type,
          onBlur = _this$props5.onBlur;
      var fixedLanguage = (0, _get2.default)(type, 'options.language');
      var mode = isSupportedLanguage(value && value.language || fixedLanguage) || 'text';
      return /*#__PURE__*/_react.default.createElement("div", {
        className: (0, _classnames.default)(_CodeInput.default.aceEditorContainer, readOnly && _CodeInput.default.readOnly)
      }, /*#__PURE__*/_react.default.createElement(_reactAce.default, {
        className: _CodeInput.default.aceEditor,
        mode: mode,
        theme: this.getTheme(),
        width: "100%",
        onChange: this.handleCodeChange,
        name: "".concat(this._inputId, "__aceEditor"),
        value: value && value.code || '',
        markers: value && value.highlightedLines ? (0, _createHighlightMarkers.default)(value.highlightedLines) : undefined,
        onLoad: this.handleEditorLoad,
        readOnly: readOnly,
        tabSize: 2,
        wrapEnabled: true,
        setOptions: _config.ACE_SET_OPTIONS,
        editorProps: _config.ACE_EDITOR_PROPS,
        onFocus: this.handleCodeFocus,
        onBlur: onBlur
      }));
    });

    _defineProperty(this, "handleLanguageFocus", () => {
      this.props.onFocus(_config.PATH_LANGUAGE);
    });

    _defineProperty(this, "handleCodeFocus", () => {
      this.props.onFocus(_config.PATH_CODE);
    });

    _defineProperty(this, "handleFilenameFocus", () => {
      this.props.onFocus(_config.PATH_FILENAME);
    });
  }

  focus() {
    var _this$editor;

    (_this$editor = this.editor) === null || _this$editor === void 0 ? void 0 : _this$editor.focus();
  }

  componentWillUnmount() {
    var _this$editor2;

    (_this$editor2 = this.editor) === null || _this$editor2 === void 0 ? void 0 : _this$editor2.removeListener('guttermousedown', this.handleGutterMouseDown);
  }

  getLanguageAlternatives() {
    var languageAlternatives = (0, _get2.default)(this.props.type, 'options.languageAlternatives');

    if (!languageAlternatives) {
      return _config.SUPPORTED_LANGUAGES;
    }

    if (!Array.isArray(languageAlternatives)) {
      throw new Error("'options.languageAlternatives' should be an array, got ".concat(typeof languageAlternatives));
    }

    return languageAlternatives.reduce((acc, _ref) => {
      var title = _ref.title,
          value = _ref.value;
      var alias = _config.LANGUAGE_ALIASES[value];

      if (alias) {
        // eslint-disable-next-line no-console
        console.warn("'options.languageAlternatives' lists a language with value \"%s\", which is an alias of \"%s\" - please replace the value to read \"%s\"", value, alias, alias);
        return acc.concat({
          title,
          value: alias
        });
      }

      if (!_config.SUPPORTED_LANGUAGES.find(lang => lang.value === value)) {
        // eslint-disable-next-line no-console
        console.warn("'options.languageAlternatives' lists a language which is not supported: \"%s\", syntax highlighting will be disabled.", value);
      }

      return acc.concat({
        title,
        value
      });
    }, []);
  }

  getTheme() {
    var preferredTheme = (0, _get2.default)(this.props.type, 'options.theme');
    return preferredTheme && _config.SUPPORTED_THEMES.find(theme => theme === preferredTheme) ? preferredTheme : _config.DEFAULT_THEME;
  }

  render() {
    var _this$props6 = this.props,
        compareValue = _this$props6.compareValue,
        value = _this$props6.value,
        type = _this$props6.type,
        level = _this$props6.level,
        readOnly = _this$props6.readOnly,
        presence = _this$props6.presence,
        onBlur = _this$props6.onBlur,
        focusPath = _this$props6.focusPath;
    var languages = this.getLanguageAlternatives().slice();

    if ((0, _has2.default)(type, 'options.language')) {
      var _codeCompareValue = PathUtils.get(compareValue, _config.PATH_CODE);

      return /*#__PURE__*/_react.default.createElement(_changeIndicators.ChangeIndicatorProvider, {
        path: _config.PATH_CODE,
        focusPath: focusPath,
        value: value === null || value === void 0 ? void 0 : value.code,
        compareValue: _codeCompareValue
      }, /*#__PURE__*/_react.default.createElement(_default2.default, {
        legend: type.title,
        description: type.description,
        level: level
      }, this.renderEditor()));
    }

    var selectedLanguage = value && value.language ? languages.find(item => item.value === value.language) : undefined;

    if (!selectedLanguage) {
      languages.unshift({
        title: 'Select language',
        value: ''
      });
    }

    var languageField = type.fields.find(field => field.name === 'language');
    var filenameField = type.fields.find(field => field.name === 'filename');
    var languageCompareValue = PathUtils.get(compareValue, _config.PATH_LANGUAGE);
    var codeCompareValue = PathUtils.get(compareValue, _config.PATH_CODE);
    var filenameCompareValue = PathUtils.get(compareValue, _config.PATH_FILENAME);
    var languagePresence = presence.filter(presenceItem => PathUtils.startsWith(_config.PATH_LANGUAGE, presenceItem.path));
    var codePresence = presence.filter(presenceItem => PathUtils.startsWith(_config.PATH_CODE, presenceItem.path));
    var filenamePresence = presence.filter(presenceItem => PathUtils.startsWith(_config.PATH_FILENAME, presenceItem.path));
    return /*#__PURE__*/_react.default.createElement(_default2.default, {
      legend: type.title,
      description: type.description,
      level: level,
      changeIndicator: false
    }, /*#__PURE__*/_react.default.createElement(_changeIndicators.ChangeIndicatorProvider, {
      path: _config.PATH_LANGUAGE,
      focusPath: focusPath,
      value: selectedLanguage === null || selectedLanguage === void 0 ? void 0 : selectedLanguage.value,
      compareValue: languageCompareValue
    }, /*#__PURE__*/_react.default.createElement(_default.default, {
      level: level + 1,
      label: (languageField === null || languageField === void 0 ? void 0 : languageField.title) || 'Language',
      presence: languagePresence
    }, /*#__PURE__*/_react.default.createElement(_ui.Select, {
      onChange: this.handleLanguageChange,
      readOnly: readOnly,
      value: (selectedLanguage === null || selectedLanguage === void 0 ? void 0 : selectedLanguage.value) || '',
      onFocus: this.handleLanguageFocus,
      onBlur: onBlur
    }, languages.map(lang => /*#__PURE__*/_react.default.createElement("option", {
      key: lang.value,
      value: lang.value
    }, lang.title))))), (0, _get2.default)(type, 'options.withFilename', false) && /*#__PURE__*/_react.default.createElement(_changeIndicators.ChangeIndicatorProvider, {
      path: _config.PATH_FILENAME,
      focusPath: focusPath,
      value: value === null || value === void 0 ? void 0 : value.filename,
      compareValue: filenameCompareValue
    }, /*#__PURE__*/_react.default.createElement(_default.default, {
      label: (filenameField === null || filenameField === void 0 ? void 0 : filenameField.title) || 'Filename',
      level: level + 1,
      presence: filenamePresence
    }, /*#__PURE__*/_react.default.createElement(_ui.TextInput, {
      name: "filename",
      value: (value === null || value === void 0 ? void 0 : value.filename) || '',
      placeholder: filenameField === null || filenameField === void 0 ? void 0 : filenameField.placeholder,
      onChange: this.handleFilenameChange,
      onFocus: this.handleFilenameFocus,
      onBlur: onBlur
    }))), /*#__PURE__*/_react.default.createElement(_changeIndicators.ChangeIndicatorProvider, {
      path: _config.PATH_CODE,
      focusPath: focusPath,
      value: value === null || value === void 0 ? void 0 : value.code,
      compareValue: codeCompareValue
    }, /*#__PURE__*/_react.default.createElement(_default.default, {
      label: "Code",
      level: level + 1,
      presence: codePresence
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: _CodeInput.default.editorContainer
    }, this.renderEditor()))));
  }

}

exports.default = CodeInput;